#include "extern.h"

#define Beta 0.0873
#define Epsilon_sigma 0.0001

#define signo(x) ((x>0.0)?1:-1)

/* -------------------------------------------------------------------------
                         Proceso de Recombinacion
  ----------------------------------------------- -------------------------- */

float OperadoresRecombinacion (int omega, int delta, int pos)
/* Operadores de recombinacion de la EE-(mu,landa) */
{
 double u, suma, salida;
 int i, padre1, padre2;

 switch (omega)
  {
   /* Recombinacion Global Intermedia */
   case 1: for (suma=0.0,i=0; i<delta; i++)
               suma += Padres[indices_recombinacion[i]].Gene[pos];
           salida=suma/delta;
           break;

   /* Recombinacion Local Intermedia */
   case 2: padre1=Randint (0, delta-1);  padre2=Randint (0, delta-1); u=Rand();
           salida=u   *Padres[indices_recombinacion[padre1]].Gene[pos] +
                 (1-u)*Padres[indices_recombinacion[padre2]].Gene[pos];
           break;

   /* Recombinacion Discreta */
   case 3: padre1=Randint (0, delta-1);
           salida=Padres[indices_recombinacion[padre1]].Gene[pos];
           break;
   }
 return (salida);
 }

/*********************************************************************************/
int Pertenece (int *indices_seleccionados, int ind_act, int n_seleccionados)
/* Devuelve 1 si el indice actual esta la lista indices_seleccionados de individuos ya seleccionados */
{
 int i;

 i=0;
 while (i<n_seleccionados)
       if (indices_seleccionados[i]==ind_act)  return (1);
       else                                    i++;
 return (0);
 }

/*************************************************************************************************/
void Recombinacion (void)
{
int n_hijo, i, j, cadena, omega, delta, n_act, princ_cadena, fin_cadena, padre;

for(n_hijo=0; n_hijo<Landa; n_hijo++)
   {
   /* Recombinacion de las tres partes del individuo:
         cadena=0 -> vector solucion x.
         cadena=1 -> vector de desviaciones estandar sigma.
         cadena=2 -> vector de angulos de inclinacion alfa. */

   for (cadena=0; cadena<=2; cadena++)
       {
       switch (cadena)
              {
              case 0: omega=Omega_x;     delta=Delta_x;     princ_cadena=0;
                      fin_cadena=n_x;                break;
              case 1: omega=Omega_sigma; delta=Delta_sigma; princ_cadena=n_x;
                      fin_cadena=n_x+n_sigma;        break;
              case 2: omega=Omega_alfa;  delta=Delta_alfa;  princ_cadena=n_x+n_sigma;
                      fin_cadena=n_x+n_sigma+n_alfa; break;
              }

      if (princ_cadena!=fin_cadena) /* Si la cadena tiene componentes, operamos */
         {
         /* Seleccion de los individuos que van a ser considerados en la recombinacion */
         if (delta==Mu)                /* si son todos, directamente copio los indices */
            for (i=0; i<delta; i++) indices_recombinacion[i]=i;
         else                         /* si no, los escogo aleatoriamente sin permitir repetidos*/
            for (i=0;i<delta;i++)                
                 do
                   indices_recombinacion[i]=Randint (0, delta-1);
                 while (Pertenece (indices_recombinacion, indices_recombinacion[i], i));

        if (omega==0)
           {
           /* Seleccionar aleatoriamente un individuo de entre los que intervienen en el proceso */
           padre=Randint (0, delta-1);
           for (j=princ_cadena; j<fin_cadena; j++)
               Hijos[n_hijo].Gene[j]=Padres[indices_recombinacion[padre]].Gene[j];
           }
        else
          /* Recombinar los componentes de la cadena actual con el operador indicado por omega */
          for (j=princ_cadena; j<fin_cadena; j++)
              Hijos[n_hijo].Gene[j]= OperadoresRecombinacion (omega, delta, j);
        }
     }
   }
 }


/* -------------------------------------------------------------------------
                        Proceso de Mutacion
  ------------------------------------------------------------------------- */

double ValorNormal (float desv)
/* Genera un valor normal con esperanza 0 y  desviacion tipica desv. */
{
 double u1, u2, salida, sqrt(), sin(), log();

 /* Se generan dos uniformes [0,1] */
 u1=Rand ();  u2=Rand ();

 /* Se calcula un valor normal (0,desv) a partir de los uniformes anteriores */

 salida=desv * sqrt (-2.0*log(u1)) * sin (2.0*PI*u2);   return (salida);

}

/*****************************************************************************/
void Mutacion (void)
 /* Proceso de mutacion de los hijos previamente generados mediante recombinacion */
 {
 int n_hijo, i, j, p, q, nq, n1, n2;
 double z0, z1, x1, x2, si, co;

 for (n_hijo=0; n_hijo<Landa; n_hijo++)
  {
   /* Mutacion de sigma */
   if (n_sigma==1) /* Si solo se usa un sigma, se adapta con Tau_1 */
    Hijos[n_hijo].Gene[n_x] *= ValorNormal (Tau_1);
   else
    {
     z0=ValorNormal (Tau_0);
     for (i=n_x;i<n_x+n_sigma;i++)
      {
       z1=ValorNormal (Tau);
       Hijos[n_hijo].Gene[i] *= exp (z1+z0);
   
       /* Si la desviacion estandar pasa a ser 0, se asigna el valor minimo
          Epsilon_sigma */
       if (Hijos[n_hijo].Gene[i]==0.0)
        Hijos[n_hijo].Gene[i]=Epsilon_sigma;
       }
     }  
     
       
   /* Mutacion de alfa */
   for (i=n_x+n_sigma;i<n_total;i++)
    {
     z0=ValorNormal (Beta);
     Hijos[n_hijo].Gene[i] += z0;

     /* Si el valor mutado se sale del intervalo [-PI,PI], se proyecta
        circularmente el valor a dicho intervalo */
     if (fabs(Hijos[n_hijo].Gene[i])>PI)
      Hijos[n_hijo].Gene[i] -= 2*PI*signo (Hijos[n_hijo].Gene[i]);
     }


   /* Mutacion de X */
   
   /* Calculo del vector no correlado de mutaciones */
   for (i=0;i<n_x;i++)
    if (n_x+i<n_x+n_sigma)
     Z[i]=ValorNormal (Hijos[n_hijo].Gene[n_x+i]);
    else /* Si no hay mas desviaciones tipicas, se usa la ultima para todos */
     Z[i]=ValorNormal (Hijos[n_hijo].Gene[n_x+n_sigma-1]);      
        
   /* Correlacion de dicho vector si estamos considerando los angulos */  
   if (n_alfa!=0)
    {
     nq=n_alfa;
     for (j=nl_alfa;j<=nm_alfa;++j)
      {
       n1=n_x-j;
       n2=n_x;
       for (i=1;i<=j;++i)
	{
         x1=Z[n1-1];
         x2=Z[n2-1];
         si=sin(Hijos[n_hijo].Gene[n_x+n_sigma+nq-1]);
         co=cos(Hijos[n_hijo].Gene[n_x+n_sigma+nq-1]);
         Z[n2-1]=x1*si + x2*co;
         Z[n1-1]=x1*co - x2*si;
	 --n2;
	 --nq;
	 }
       } 
     }
    
   /* Mutacion final de X */ 
   for (i=0;i<n_x;i++)
    {
     Hijos[n_hijo].Gene[i] += Z[i];
     if (Hijos[n_hijo].Gene[i]<MIN)
      Hijos[n_hijo].Gene[i]=MIN;
     if (Hijos[n_hijo].Gene[i]>MAX)
      Hijos[n_hijo].Gene[i]=MAX;
     }
   }
 }


/* -------------------------------------------------------------------------
	                   Proceso de Seleccion
  ------------------------------------------------------------------------- */

int compara (const void *a, const void *b)
/* Funcion para comparar dos valores empleada por el algoritmo qsort, el cual
   se encarga de ordenar la poblacion genetica de mejor a peor fitness.
   Depende de la macro BETTER, que se encuentra en define.h, la cual indica
   si el problema es de maximizacion o minimizacion */

{
 if (BETTER ((* (STRUCTURE *) a).fitness, (* (STRUCTURE *) b).fitness))
  return (-1);
 else
  return (1);
 }



void SeleccionComa (void)
{
 int i, j, temp;

 /* Evaluo los Landa hijos obtenidos */
 for (i=0; i<Landa; i++)
  Hijos[i].fitness=eval (Hijos[i].Gene, n_x); Trials++;

 /* Ordeno la poblacion de hijos generados */
 qsort ((void *) Hijos,Landa,sizeof (STRUCTURE),compara);
 
 /* Selecciono los Mu mejores para formar la siguiente poblacion de padres */
 for(i=0; i<Mu; i++)
  for (j=0;j<n_total;j++)
   Padres[i].Gene[j]=Hijos[i].Gene[j];
   
}


/* -------------------------------------------------------------------------
   	                Estrategia de Evolucion-(Mu,Landa)
  ------------------------------------------------------------------------- */

void EE_MuComaLanda (void)

{
 int i, j;

 InicializaPadres ();
 for(i=1;i<=n_gen_ee;i++)
  {
   Recombinacion (); 
   Mutacion (); 
   SeleccionComa ();
    
   /* Compruebo si he encontrado una mejor solucion global */
   if (BETTER(Hijos[0].fitness, Best_guy.fitness))
    {
     for (j=0;j<n_total;j++)
      Best_guy.Gene[j]=Hijos[0].Gene[j];
     Best_guy.fitness=Hijos[0].fitness;
     Best_guy.Gen=i;
     }

   /* Visualizacion de estadisticas */
   if (i==1 || i%20==0)
    printf ("\n Gen. Actual: %d. Mejor Individuo: %e. Gen. Mejor: %d",
            i, Best_guy.fitness, Best_guy.Gen);
   }
   
 printf ("\n\nMejor Individuo:\n (");
 for (j=0;j<n_x-1;j++)
  printf ("%e, ", Best_guy.Gene[j]);
 printf ("%e)\n\n", Best_guy.Gene[n_x-1]);
 printf ("Fitness: %e. Gen. Mejor: %d\n",Best_guy.fitness, Best_guy.Gen);
 }

