#include "cmaescb.h"

using namespace std;

#include <vector>
#include <numeric>
#include <queue>

#include <algorithm>
#include <iterator>


/**
 * Esta funcin permite obtener la matriz indicada manteniendo todas las variables en orden
 */
MyReturnMatrix xintobounds(const ColumnVector   &mat, Real lower, Real upper, vector<int> *corrected) {
    int n=mat.Nrows();
    Real lowers[n];
    Real uppers[n];
    int i;

    for (i = 0; i < n; i++) {
	lowers[i] = lower;
	uppers[i] = upper;
    }

    return xintobounds(mat, lowers, uppers, corrected);
}


/**
 * Esta funcin permite obtener la matriz indicada manteniendo todas las variables en orden
 */
MyReturnMatrix xintobounds(const ColumnVector   &mat, Real *lower, Real *upper, vector<int> *corrected) {
    Real val;
    int posi, i;
    int n= mat.Nrows();

    ColumnVector mat_corrected(n);

    for (i = 0; i < n; i++) {
       val = mat[i];
       posi = i;

       if (val < lower[i]) {
	  val = lower[i];
	  if (corrected != NULL) {
	     corrected->push_back(posi);
	  }
       }
       else if (val > upper[i]) {
	  val = upper[i];

	  if (corrected != NULL) {
	     corrected->push_back(posi);
	  }
       }

       mat_corrected[i] = val;
    }

    mat_corrected.Release();
    return mat_corrected;
}

void CMAESBnd::setParam(int lambda, double mueff, double sigma, MyMatrix &C) {
   m_lambda = lambda;
   m_mueff = mueff;
   m_sigma = sigma;
   m_dfithist_size = 20+(3*m_ndim)/lambda;
   m_dfithist.push(1.0);

   DiagonalMatrix DiagC(m_ndim);
   DiagC <<C;
   copyColumn(DiagC, m_diagC);
}

CMAESBnd::CMAESBnd(double *lbound, double *ubound, int N, int check, int avanzed) :
m_diagC(N), m_scale(N), m_weights(N), m_dfithist() {
    int i;

    m_ndim = N;
    m_validfitval = 0;
    m_iniphase = true;
    m_isactive = (check && lbound < ubound);
    m_numapplied = 0;

    if (!m_isactive) {
        m_lbounds = m_ubounds = NULL;
	return;
    }

    m_cmaes_criterion = avanzed;
    m_lbounds = new double[N];
    m_ubounds = new double[N];

    for (i = 0; i < N; ++i) {
       m_lbounds[i] = lbound[i];
       m_ubounds[i] = ubound[i];
    }

    m_weights = 0; // weights for bound penalty
    Real meanC = mean(m_diagC);
    m_scale = m_diagC/meanC;
}

CMAESBnd::~CMAESBnd(void) {
   if (m_lbounds != NULL) {
      delete []m_lbounds;
      m_lbounds = NULL;
   }
   if (m_ubounds != NULL) {
      delete []m_ubounds;
      m_ubounds = NULL;
   }

   m_isactive = false;
}

void CMAESBnd::evalSols(ColumnVector &xmean, MyMatrix &arx, tCMAESEval eval, void *eval_options, MyMatrix &arxvalid, RowVector &fitness_raw, RowVector &fitness_sel) {
   int lambda = arx.Ncols();
   int i;

try {
   vector<int> v_ti;
   v_ti.reserve(m_ndim);

   // Incremento el contador
   m_numapplied += 1;
//   cout <<"CMAESBnd::eval the "  <<m_numapplied <<" solutions" <<endl;

    // Si est activado simplemente 'corta' la evaluacin
    if (!m_isactive) {
       arxvalid = arx;
    }
    else {
       for (i = 1; i <= lambda; ++i) {
	    ColumnVector arx_i = arx.Column(i);
	    arxvalid.Column(i) = xintobounds(arx_i, m_lbounds, m_ubounds, &v_ti);
       }
	
    }

    for (i = 1; i <= lambda; ++i) {
       ColumnVector arxvalid_i = arxvalid.Column(i);
       fitness_raw[i-1] = eval(arxvalid_i.Store(), m_ndim, eval_options);
    }

    // Inicialmente fitness_sel = fitness_raw
    fitness_sel = fitness_raw;

    // Cambio arx 
    if (!m_isactive || !m_cmaes_criterion) {
	return;
    }

    // Slo si se sigue el criterio fitness_sel se diferencia de fitness
    // (penalizando las soluciones que se salen fuera)
    int perc[2] = {25,75};

    // Get delta fitness values
    RowVector r_val = myprctile(fitness_raw, perc, 2);
    Real val = (r_val[1]-r_val[0]) / m_ndim / mean(m_diagC) / pow2(m_sigma);

    if (val == 0) { // happens if all points are out of bounds
       val = min_positive(m_dfithist);
    }
    else if (m_validfitval == 0) { // first sinsible var
       m_dfithist.pop();
       m_validfitval = 1;
    }

    // Actualizo el histograma
    if (m_dfithist.size() == m_dfithist_size) {
       m_dfithist.pop();
    }

    m_dfithist.push(val);

    RowVector dfithist(m_dfithist.size());
    copyRow(m_dfithist, dfithist);

    // Actualizo los pesos y la escala
    m_weights = DotVectors(m_scale, m_weights);

    Real mean_d = mean(log(m_diagC));
    m_scale << exp(0.1*mean_d)*pow(m_diagC, 0.9);

    // cout <<"bnd_scale After :\n" <<m_scale <<endl;
    m_scale = m_scale / exp(mean(log(m_scale))); // prod is 1 initially
    // cout <<"bnd_scale Rescale :\n" <<m_scale <<endl;

    m_weights = DivVectors(m_weights, m_scale);
    // cout <<"bnd_weights:\n" <<m_weights <<endl;

    // Verifico que xmean est dentro del rango
    v_ti.clear();
    ColumnVector tx = xintobounds(xmean, m_lbounds, m_ubounds, &v_ti);

    // cout <<"tx : " <<tx <<endl;

    // Set initial weights
    if (m_iniphase) {
       // cout <<"initphase" <<endl;
       // cout <<"v_ti.size : " <<v_ti.size() <<endl;

       // La primera vez que se sale de la frontera actualiza los pesos
       if (v_ti.size() ) {
	  // copy(v_ti.begin(), v_ti.end(), ostream_iterator<int> (cout,","));
try {
	  // cout <<endl;
	  Real hist_median =  median(dfithist);
	  m_weights = 2.0002* hist_median;
	  // cout <<"bnd_weights init:\n" <<m_weights <<endl;
	  m_weights = DivVectors(m_weights, m_scale);
	  // cout <<"initphase: bnd_weights:\n" <<m_weights <<endl;
}
catch (Exception &e) {
    cerr <<"Dentro de iniphase : " <<e.what() <<endl;
}
	  if (m_validfitval && m_numapplied > 2) {
	     m_iniphase = false;
	  }
       }
    }

    // Increase/decrease weights
    if  (v_ti.size() ) { // any coordinate of xmean out of bounds
       // judge distance of xmean to boundary
       tx = xmean - tx;
       Real umbral_out = 3*max(1.0,sqrt(m_ndim)/m_mueff)*m_sigma;
       Real increm = pow(1.1, max(1.0, m_mueff/10.0/m_ndim));
       vector<int>::iterator item;
       int posi;

       for (item = v_ti.begin(); item != v_ti.end(); item++) {
	  posi = *item;

	  if (tx[posi] > umbral_out*sqrt(m_diagC[posi])) {
	     m_weights *= increm;
	  }
       }
    }

    // cout <<"calculando dif" <<endl;
    Matrix dif = arxvalid - arx;
    // cout <<"dif : \n" <<dif <<endl;

    // cout <<"pow2_m(dif): \n" <<pow2_m(dif) <<endl;
    RowVector arpenalty = m_weights.t() * pow2_m(dif);
    // cout << "arpenalty : " << arpenalty <<endl;
    // Aplica penalizacin
    fitness_sel = fitness_raw + arpenalty;
}
catch (Exception &e) {
    cerr <<"Excepcin recuperada" <<endl;
    cerr <<e.what() <<endl;
}
}


