#ifndef _CMAESCB_H

#define _CMAESCB_H 1

#include <queue>

#include "newutil.h"

/**
 * Esta funcin permite adaptar la funcin de evaluacin de CMAES
 *
 * (Siempre debe de minimizar). 
 *
 * @param sol Vector de parmetros.
 * @param ndim Tamao del vector
 * @param eval_params Parmetros adiciones de evaluacin
 */
typedef double(*tCMAESEval)(Real *sol, int ndim, void *eval_params);  

MyReturnMatrix xintobounds(const ColumnVector   &mat, Real lower, Real upper, vector<int> *corrected); 

MyReturnMatrix xintobounds(const ColumnVector   &mat, Real *lower, Real *upper, vector<int> *corrected); 

/**
 * Esta clase permite encapsular la gestin de las fronteras
 */
class CMAESBnd {
public:
   CMAESBnd(double *lbound, double *ubound, int n, int check=true, int avanzed=false);
   ~CMAESBnd(void);

/**
 * Este mtodo permite iniciar los valores asociados a la BL
 *
 * @param lambda valor lambda.
 * @param mueff.
 * @param sigma.
 * @param C matrix de covarianza.
 */
void setParam(int lambda, double mueff, double sigma, MyMatrix &C);

/**
 * Este mtodo permite, dado un individuo devolver dos fitness. Un fitness real y el fitness utilizado para obtener la media de los lambda mejores. 
 *
 * (Nota: Si is_active es falso no realiza nada).
 *
 * Si es avanzado aplica el criterio de fitness.
 * Si no es avanzado simplementen devolver en 
 * arx la solucin 'cortada' por los bordes, y
 * fitness == fitness_sel. 
 *
 * En todo caso aplica las accin correspondiente, oculta al algoritmo.
 *
 * @param arx solucin actual a verificar los bordes.
 *a
 * @param eval Funcin de evaluacin. 
 * @param ls Referencia al objeto de BL a utilizar.
 * @param arxvalid Vede las soluciones finales (dentro de las fronteras).
 * @param fitness Fitness obtenido.
 * @param fitness_sel Fitness utilizado para la seleccin.
 *
 */
void evalSols(ColumnVector &xmean, MyMatrix &arx, tCMAESEval eval, void *eval_options, MyMatrix &arxvalid, RowVector &fitness_raw, RowVector &fitness_sel);

private:
   /**
    * @var Dimensin
    */
   int m_ndim;

   /**
    * @var Lambda
    */
   int m_lambda;

   /**
    * @var mueff
    */
   double m_mueff;

   /**
    * @var Diagonal de la Matrix de Covarianza
    */
   ColumnVector m_diagC;

   /**
    * @var Valor sigma actual
    */
   double m_sigma;

   /**
    * @var indica si se debe de comparar las fronteras
    */
    bool m_isactive;

    /**
     * @var lmite inferior
     */
    double *m_lbounds;

    /**
     * @var lmite inferior
     */
    double *m_ubounds;


    /**
     * @var Si est a true sigue el criterio de CMAES avanzado (admite puntos
     * fuera de los bordes, y slo reacciona cuando la media se encuentra fuera
     * de los mismos).
     * Si est a false simplemente 'corta' las soluciones
     */
    bool m_cmaes_criterion;

private:
    /**
     * Vector escala de bordes
     */
    ColumnVector m_scale;

    /**
     * Vector pesos de los bordes
     */
    ColumnVector m_weights;

    /**
     * @varHistograma de fronteras
     */
    queue<Real> m_dfithist;
    
    /**
     * Su tamao actual
     */
    unsigned int m_dfithist_size;

    // delta fit for setting weights
    int m_validfitval;
    bool m_iniphase;

    // Cuento el nmero de veces aplicado
    unsigned int m_numapplied;
};

#endif
