#ifndef _RANDOM_H

#define _RANDOM_H

#define MASK 2147483647
#define PRIME 65539
#define SCALE 0.4656612875e-9

/**
 * @class Random
 *
 * Esta clase permite generar nmeros aleatorios
 *
 */
class Random {
   public:
       /**
	* inicia el valor de la semilla
	*/
       static void setSeed(unsigned long seed);

       /**
	* Genera aleatoriamente un nmero entre 0 y 1
	*
	* @return Un nmero aleatorio entre 0 y 1
	*/
       static double rand(void);

       /**
	* Devuelve la semilla actual (para almacenarla junto con los
	* resultados
	* @return El valor de la semilla
	*/
       static unsigned long getSeed(void);

       /**
	* Genera aleatoriamente un nmero entero entre low y high
	*
	* @param low Valor mnmo
	* @param high Valor mximo
	* 
	* @return Un nmero aleatorio entre low y high
	*/
       static int randint(unsigned low, unsigned high) {
	  double random;

	  random = Random::rand();
	  return ( (int) (low + (high-low+1)*random) );
       }

       /**
	* Genera aleatoriamente un nmero real entre low y high
	*
	* @param low Valor mnimo
	* @param high Valor mximo
	* 
	* @return Un nmero aleatorio entre low y high
	*/
       static double randreal(double low, double high) {
	  return (low + (high-low)*Random::rand());
       }


       /**
	* Genera un nmero real siguiendo una distribucin normal centrado 
	* en 0 y con 'desv' de desviacin tpica
	*
	* @param desv Desviacin tpica
	* @return Un nmero aleatorio entre 0 y 1
	*/
	static double normal(double desv);
};

#endif
