#include <iostream>

#include "cmaes.h"
#include <math.h>
#include <assert.h>

/**
 * Funcin de evaluacin a minimizar.
 *
 * @param sol vector solucin a evaluar
 * @param ndim tamao de la dimensin
 * @param eval_params Vector opciones (permite pasarle informacin a evalSphere, incluso un tipo LocalSearch
 * tan complejo como se quiera)
 * En este ejemplo se ignorar
 *
 * @return el valor de fitness a minimizar
 */
double evalSphere(Real *sol, int ndim, void *eval_params) {
    double sum;
    int i;
    
    // El optimo ser la solucin (0, 1, 2, 3, 4, ..., ndim-1)
    for (sum = 0, i = 0; i < ndim; ++i) {
	sum += fabs(sol[i]-i);
    }

    return sum;
}

int main(void) {
    int ndim = 10; /* Cambiar por dimensin del problema */
    Real sol[ndim];
    void *eval_options = NULL;
    LocalSearchParamsPtr params;
    double initial_sigma;

    // Inicio en cero
    fill(sol, sol+ndim, 0);

    // Valores extremos
    double minvalue = -ndim; /* Cambiar por min intervalo variacin: I_I */
    double maxvalue = -ndim; /* Cambiar por max intervalo variacin: I_D */

    CMAESC cmaes(evalSphere, ndim, minvalue, maxvalue);

    // Obtengo el valor inicial de todo el dominio
    initial_sigma = (maxvalue-minvalue)/3;

    // Obtengo los parmetros iniciales de la BL
    params = cmaes.getInitParams(initial_sigma);

    // Evalo la solucin
    double fitness = evalSphere(sol, ndim, eval_options);

    cout <<"Fitness antes de BL : " <<fitness <<endl;

    cmaes.setMaxEval(10000*ndim);
    cmaes.apply(sol, &fitness, eval_options, params);

    // En params obtiene el estado actual de la bsqueda, por si se quisiese guardar

    // En sol obtiene el optimo
    cout <<"Fitness despus de BL : " <<fitness <<endl;

    assert(fitness == evalSphere(sol, ndim, eval_options) );
}
